/*  This is the Arduino sketch that is pre-programmed into the PRIZM controllers for Mr. Robot 2.
 *  The example sketch requires the TETRIX PRIZM and Tele-Op controller Arduino libraries. Both 
 *  are available for download at the Pitsco Education website: www.pitsco.com
 *  Hardware: Mr. Robot, PRIZM controller, Tele-Op module + SONY PS4 gaming controller
 *  By: Paul W. Uttley
 *  01/15/19
 * 
 */

#include <PRIZM.h>      // TETRIX PRIZM controller library
#include <TELEOP.h>     // TETRIX Tele-Op module library
PRIZM prizm;            // create a PRIZM object named prizm
PS4 ps4;                // create a Tele-Op module object for the PS4 controller

int Left_Elbow  = 50;   // variables and default positions for Mr. Robot's proportional controlled servo motors
int Right_Elbow = 50;;
int Grippers    = 90;
int Head        = 90;

int Right_Motor = 0;    // right foot track
int Left_Motor  = 0;    // left foot track

int Sonic_Sensor;       // variable for ultrasonic sensor value


void setup() {
 
  prizm.PrizmBegin();
  prizm.setMotorInvert(1,1);            // invert the direction of DC motor 1

  prizm.setServoSpeed(1,50);            // LEFT Shoulder is servo 1
  prizm.setServoSpeed(4,Left_Elbow);    // LEFT Elbow is servo 4

  prizm.setServoSpeed(2,50);            // RIGHT Shoulder is servo 2
  prizm.setServoSpeed(5,Right_Elbow);   // RIGHT Elbow is servo 5

  prizm.setServoPosition(6,Grippers);   // both grippers are on servo channel 6
  prizm.setServoPosition(3,Head);       // head is servo 3
  
  ps4.setDeadZone(LEFT,10);             // set the LEFT Joystick dead zone range to +/- 10 
  ps4.setDeadZone(RIGHT,10);            // set the RIGHT Joystick dead zone range to +/- 10 
  

}

void loop() {

  ps4.getPS4();         // Get input data and write output data to PS4 controller
  
  if(ps4.Connected){    // if PS4 Controller is connected, send control data to PRIZM

     twoStickDrive();


     
// ============ LEFT ARM ===============================
      if(ps4.Button(L1)) {prizm.setServoPosition(1,20);}    // if pressed
      if(!ps4.Button(L1)){prizm.setServoPosition(1,160);}   // if released
      Left_Elbow = ps4.Button(L2T);
      Left_Elbow = map(Left_Elbow,0,255,140,40);            // map the 0 - 255 value to a servo range of 140 - 40
      prizm.setServoPosition(4,Left_Elbow);

// ============ RIGHT ARM ===============================
      if(ps4.Button(R1)) {prizm.setServoPosition(2,160);}   // if pressed
      if(!ps4.Button(R1)){prizm.setServoPosition(2,20);}    // if released
      Right_Elbow = ps4.Button(R2T);
      Right_Elbow = map(Right_Elbow,0,255,40,140);          // map the 0 - 255 value to a servo range of 140 - 40
      prizm.setServoPosition(5,Right_Elbow);      

// ============ GRIPPERS ===========================

      if(ps4.Button(CIRCLE)){
        Grippers += 5;      // increments of 5
        Grippers = constrain(Grippers, 40, 140);            // keep value between 40 and 140
        prizm.setServoPosition(6,Grippers);
        }

       if(ps4.Button(SQUARE)){
        Grippers -= 5;      // decrements of 5
        Grippers = constrain(Grippers, 40, 140);            // keep value between 40 and 140
        prizm.setServoPosition(6,Grippers);
        }

// ============= HEAD ============================

      if(ps4.Button(LEFT)){
        Head += 5;          // increments of 5
        Head = constrain(Head, 30, 150);                    // keep value between 30 and 150
        prizm.setServoPosition(3,Head);
        }

      if(ps4.Button(RIGHT)){
        Head -= 5;          // decrements of 5
        Head = constrain(Head, 30, 150);                    // keep value between 30 and 150
        prizm.setServoPosition(3,Head);
        }  
   
     if(ps4.Button(UP)){
        Head = 90;    
        prizm.setServoPosition(3,Head);                     // move head to center
        }              

        
// ============ Read Sonic_Sensor Sensor =================

     Sonic_Sensor = prizm.readSonicSensorCM(2);
         
     if (Sonic_Sensor < 40 && Sonic_Sensor != 0){Sensor_behavior();}   // if an object is sensed with 40 centimeters, and not equal to zero, go do sensor behavior function
                                                                       // zero means that sensor is unplugged

                                                          
  }  else {   // if PS4 not connected, or becomes disconnected, stop robot
        
      prizm.setMotorPower(1,0);       // shutdown motors if connection to PS4 is lost
      prizm.setMotorPower(2,0);       // shutdown motors if connection to PS4 is lost
      prizm.setServoPosition(1,160);  // left shoulder
      prizm.setServoPosition(2,20);   // right shoulder
      prizm.setServoPosition(3,90);   // head
      prizm.setServoPosition(4,140);  // left elbow
      prizm.setServoPosition(5,40);   // right elbow
      prizm.setServoPosition(6,90);   // grippers
      
  }
}



void twoStickDrive(){
  
      Right_Motor = ps4.Motor(RY);          //assign right Y axis joystick value to right DC motor power variable
      Left_Motor  = ps4.Motor(LY);          //assign left Y axis joystick value to left DC motor power variable
      prizm.setMotorPower(1,Right_Motor);   //set speed and direction of motor 1 using LEFT Y axis joystick
      prizm.setMotorPower(2,Left_Motor);    //set speed and direction of motor 2 using RIGHT Y axis joystick
 
}

void Sensor_behavior(){
  ps4.setRumble(FAST);                      // vibrate PS4 rumble motor
  ps4.setLED(RED);                          // set PS4 light bar to red
  prizm.setMotorPowers(0,0);                // stop both DC track motors
  prizm.setServoPosition(3,90);             // turn head to center
  prizm.setServoSpeed(3,50);                // change head servo speed to 50%
  delay(500);                               // wait 1/2 second
  
  while(prizm.readSonicSensorCM(2) < 40 && prizm.readSonicSensorCM(2)!= 0){       // while the sensor detects something closer than 40cm, shake head side to side
                                                                                  // or, zero means sensor is unplugged so don't execute behavior
       prizm.setServoPosition(3,30);            // turn head                      
       delay(500);
       prizm.setServoPosition(3,150);           // turn head
       delay(500);
       
    }    
  
       ps4.setRumble(STOP);             // stop PS4 rumble motor
       ps4.setLED(GREEN);               // set PS4 light bar to green
       prizm.setServoPosition(3,90);    // turn head to center
       prizm.setServoSpeed(3,100);      // change head servo speed back to 100%
}
